﻿using Hotline.Application.FlowEngine;
using Hotline.Article;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.Permissions;
using Hotline.Repository.SqlSugar.Extensions;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.Article;
using Hotline.Share.Dtos.FlowEngine;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Article;
using MapsterMapper;
using Microsoft.AspNetCore.Mvc;
using Microsoft.OpenApi.Writers;
using XF.Domain.Authentications;
using XF.Domain.Exceptions;
using XF.Domain.Repository;
using XF.Utility.EnumExtensions;

namespace Hotline.Api.Controllers
{

    public class ArticleController: BaseController
    {
        private readonly IRepository<Bulletin> _bulletinRepository;
        private readonly IMapper _mapper;
        private readonly ISystemDomainService _systemDomainService;
        private readonly ISystemOrganizeRepository _organizeRepository;
        private readonly IWorkflowApplication _workflowApplication;
        private readonly IRepository<Circular> _circularRepository;
        private readonly ISessionContext _sessionContext;
        private readonly IRepository<CircularRecord> _circularRecordRepository;

        public ArticleController(IRepository<Bulletin> bulletinRepository,IMapper mapper, ISystemDomainService systemDomainService, ISystemOrganizeRepository organizeRepository, IWorkflowApplication workflowApplication, IRepository<Circular> circularRepository,ISessionContext sessionContext,IRepository<CircularRecord> circularRecordRepository)
        {
            _bulletinRepository = bulletinRepository;
            _mapper = mapper;
            _systemDomainService = systemDomainService;
            _organizeRepository = organizeRepository;
            _workflowApplication = workflowApplication;
            _circularRepository = circularRepository;
            _sessionContext = sessionContext;
            _circularRecordRepository = circularRecordRepository;
        }
        #region 通知
        /// <summary>
        /// 查询通知列表
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.QueryCircularList)]
        [HttpGet("circular/query")]
        public async Task<PagedDto<CircularDto>> QueryCircularList([FromQuery] QueryCircularListRequestDto dto)
        {
            var (total,items) = await _circularRepository.Queryable()
                .Includes(x=>x.CircularReadGroups)
                .WhereIF(!string.IsNullOrEmpty(dto.CircularTypeId), d => d.CircularTypeId == dto.CircularTypeId)
                .WhereIF(!string.IsNullOrEmpty(dto.Title), d => d.Title.Contains(dto.Title))
                .WhereIF(dto.CircularTimeStart.HasValue, d => d.CircularTime >= dto.CircularTimeStart)
                .WhereIF(dto.CircularTimeEnd.HasValue, d => d.CircularTime <= dto.CircularTimeEnd)
                .OrderByDescending(d => d.CreationTime)
                .ToPagedListAsync(dto.PageIndex, dto.PageSize, HttpContext.RequestAborted);
            return new PagedDto<CircularDto>(total, _mapper.Map<IReadOnlyList<CircularDto>>(items));
        }

        /// <summary>
        /// 通知详情（内部）
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [Permission(EPermission.CircularEntity)]
        [HttpGet("circular/entity/{id}")]
        public async Task<CircularDto> CircularEntity(string id)
        {
            var model = await _circularRepository.Queryable()
                .Includes(x => x.CircularReadGroups)
                .FirstAsync(x => x.Id == id, HttpContext.RequestAborted);
            return _mapper.Map<CircularDto>(model);
        }

        /// <summary>
        /// 新增通知
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.AddCircular)]
        [HttpPost("circular/add")]
        public async Task AddCircular([FromBody] CircularFlowDto dto)
        {
            var model = _mapper.Map<Circular>(dto.Data);
            model.CircularState = ECircularState.InReview;
            model.ReadedNum = 0;
            model.NeedReadNum = model.CircularReadGroups.Count;
            var id = await _circularRepository.AddAsync(model, HttpContext.RequestAborted);
            try
            {
                var startDto = _mapper.Map<StartWorkflowDto>(dto.Workflow);
                startDto.DefinitionModuleCode = WorkflowModuleConsts.CircularApply;
                startDto.Title = model.Title;
                await _workflowApplication.StartWorkflowAsync(startDto, id, HttpContext.RequestAborted);
            }
            catch (Exception ex)
            {
                await _circularRepository.RemoveAsync(id, false, HttpContext.RequestAborted);
                throw new UserFriendlyException($"新增通知流程失败！,{ex.Message}", "新增通知流程失败");
            }
        }

        /// <summary>
        /// 列表页面基础信息
        /// </summary>
        /// <returns></returns>
        [HttpGet("circular/listbasedata")]
        public async Task<object> CircularBaseData()
        {
            var rsp = new
            {
                CircularType = _systemDomainService.GetSysDicDataByCodeAsync(SysDicTypeConsts.CircularType)
            };
            return rsp;
        }

        /// <summary>
        /// 新增页面基础数据
        /// </summary>
        /// <returns></returns>
        [HttpGet("circular/addbasedata")]
        public async Task<object> CircularAddBaseData()
        {
            var rsp = new
            {
                CircularType = _systemDomainService.GetSysDicDataByCodeAsync(SysDicTypeConsts.CircularType),
                OrgsOptions = await _organizeRepository.GetOrgJson(),
            };
            return rsp;
        }

        /// <summary>
        /// 通知阅读（内部）
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("circular/read/{id}")]
        public async Task<CircularDto> CircularRead(string id)
        {
            var model = await _circularRepository.Queryable()
                .Includes(x => x.CircularReadGroups)
                .FirstAsync(x => x.Id == id, HttpContext.RequestAborted);

            //更新阅读信息
            if (model.CircularType == ECircularType.Person)
            {
                //个人阅读
            }
            else
            {
                //部门阅读
            }

            return _mapper.Map<CircularDto>(model);
        }

        /// <summary>
        /// 获取小红点计数
        /// </summary>
        /// <returns></returns>
        [HttpGet("megcount")]
        public async Task<CircularRecordDto> QueryMegCount()
        {
            var list = await _circularRecordRepository.Queryable().Where(x => x.UserId == _sessionContext.UserId || x.OrgId == _sessionContext.OrgId).ToListAsync();
            if (list!=null && list.Count>0)
            {
                var rsp = new CircularRecordDto();
                rsp.OrgCount = list.FirstOrDefault(x => x.CircularType == ECircularType.Org)?.RecordCount ?? 0;
                rsp.PersonCount = list.FirstOrDefault(x => x.CircularType == ECircularType.Person)?.RecordCount ?? 0;
                rsp.SumCount = rsp.OrgCount + rsp.PersonCount;
            }
            return new CircularRecordDto() { SumCount = 0, OrgCount = 0, PersonCount = 0 };
        }


        /// <summary>
        /// 查询通知流程开启参数
        /// </summary>
        /// <returns></returns>
        [HttpGet("circular/startflow")]
        public async Task<DefinedStepDto> GetCircularFlowStartOptions()
        {
            return await _workflowApplication.GetStartOptionsAsync(WorkflowModuleConsts.CircularApply, HttpContext.RequestAborted);
        }


        #endregion


        #region 公告

        /// <summary>
        /// 查询公告列表
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.QueryBulletinList)]
        [HttpGet("bulletin/query")]
        public async Task<PagedDto<BulletinDto>> QueryBulletinList([FromQuery]QueryBulletinListRequestDto dto)
        {
            var (total,items) = await _bulletinRepository.Queryable()
                .WhereIF(!string.IsNullOrEmpty(dto.BulletinTypeId), d => d.BulletinTypeId == dto.BulletinTypeId)
                .WhereIF(!string.IsNullOrEmpty(dto.Title), d => d.Title.Contains(dto.Title))
                .WhereIF(dto.BulletinTimeStart.HasValue, d => d.BulletinTime >= dto.BulletinTimeStart)
                .WhereIF(dto.BulletinTimeEnd.HasValue, d => d.BulletinTime <= dto.BulletinTimeEnd)
                .OrderByDescending(d => d.CreationTime)
                .ToPagedListAsync(dto.PageIndex, dto.PageSize, HttpContext.RequestAborted);

            return new PagedDto<BulletinDto>(total, _mapper.Map<IReadOnlyList<BulletinDto>>(items));
        }

        /// <summary>
        /// 公告详情（内部）
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [Permission(EPermission.BulletinEntity)]
        [HttpGet("bulletin/entity/{id}")]
        public async Task<BulletinDto> BulletinEntity(string id)
        {
            var model = await _bulletinRepository.Queryable()
                .FirstAsync(x=>x.Id == id, HttpContext.RequestAborted);
            return _mapper.Map<BulletinDto>(model);
        }

        /// <summary>
        /// 新增公告
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [Permission(EPermission.AddBulletin)]
        [HttpPost("bulletin/add")]
        public async Task AddBulletin([FromBody]BulletinFlowDto dto)
        {
            var model = _mapper.Map<Bulletin>(dto.Data);
            model.BulletinState = Share.Enums.Article.EBulletinState.InReview;
            model.ReadedNum = 0;
            var id = await _bulletinRepository.AddAsync(model, HttpContext.RequestAborted);
            try
            {
                var startDto = _mapper.Map<StartWorkflowDto>(dto.Workflow);
                startDto.DefinitionModuleCode = WorkflowModuleConsts.BulletinApply;
                startDto.Title = model.Title;
                await _workflowApplication.StartWorkflowAsync(startDto, id, HttpContext.RequestAborted);
            }
            catch (Exception ex)
            {
                await _bulletinRepository.RemoveAsync(id, false, HttpContext.RequestAborted);
                throw new UserFriendlyException($"新增公告流程失败！,{ex.Message}", "新增公告流程失败");
            }
        }

        /// <summary>
        /// 列表页面基础数据
        /// </summary>
        /// <returns></returns>
        [HttpGet("bulletin/listbasedata")]
        public async Task<object> BulletinListBaseData()
        {
            var rsp = new
            {
                BulletinType = _systemDomainService.GetSysDicDataByCodeAsync(SysDicTypeConsts.BulletinType)
            };
            return rsp;
        }


        /// <summary>
        /// 新增页面基础数据
        /// </summary>
        /// <returns></returns>
        [HttpGet("bulletin/addbasedata")]
        public async Task<object> BulletinAddBaseData()
        {
            var rsp = new
            {
                BulletinType = _systemDomainService.GetSysDicDataByCodeAsync(SysDicTypeConsts.BulletinType),
                PushRanges = EnumExts.GetDescriptions<EPushRange>(),
                OrgsOptions = await _organizeRepository.GetOrgJson(),
            };
            return rsp;
        }


        /// <summary>
        /// 查询公告流程开启参数
        /// </summary>
        /// <returns></returns>
        [HttpGet("bulletin/startflow")]
        public async Task<DefinedStepDto> GetBulletinFlowStartOptions()
        {
            return await _workflowApplication.GetStartOptionsAsync(WorkflowModuleConsts.BulletinApply, HttpContext.RequestAborted);
        }


        #endregion

       

    }
}
