﻿using DotNetCore.CAP;
using Hotline.Article;
using Hotline.CallCenter.Tels;
using Hotline.FlowEngine.Notifications;
using Hotline.FlowEngine.WorkflowModules;
using Hotline.FlowEngine.Workflows;
using Hotline.KnowledgeBase;
using Hotline.Orders;
using Hotline.Repository.SqlSugar.Orders;
using Hotline.Settings;
using Hotline.Share.Dtos;
using Hotline.Share.Dtos.FlowEngine.Workflow;
using Hotline.Share.Dtos.Order;
using Hotline.Share.Enums.Order;
using Hotline.Share.Mq;
using MapsterMapper;
using MediatR;
using XF.Domain.Exceptions;
using XF.Domain.Repository;

namespace Hotline.Application.Handlers.FlowEngine;

public class EndWorkflowHandler : INotificationHandler<EndWorkflowNotify>
{

    private readonly IKnowledgeDomainService _knowledgeDomainService;
    private readonly IOrderDomainService _orderDomainService;
    private readonly ITelDomainService _telDomainService;
    private readonly IWorkflowDomainService _workflowDomainService;
    private readonly ISystemOrganizeRepository _orgRepository;
    private readonly IOrderRepository _orderRepository;
    private readonly ICapPublisher _capPublisher;
    private readonly IMapper _mapper;
    private readonly IRepository<OrderVisitDetail> _orderVisitedDetailRepository;
    private readonly IRepository<OrderScreen> _orderScreenRepository;
    private readonly IRepository<Bulletin> _bulletinRepository;
    private readonly IRepository<Circular> _circularRepository;

    public EndWorkflowHandler(
        IKnowledgeDomainService knowledgeDomainService,
        IOrderDomainService orderDomainService,
        ITelDomainService telDomainService,
        IWorkflowDomainService workflowDomainService,
        ISystemOrganizeRepository orgRepository,
        IOrderRepository orderRepository,
        ICapPublisher capPublisher,
        IMapper mapper,
        IRepository<OrderVisitDetail> orderVisitedDetailRepository,
        IRepository<OrderScreen> orderScreenRepository,
        IRepository<Bulletin> bulletinRepository,
        IRepository<Circular> circularRepository
        )
    {
        _knowledgeDomainService = knowledgeDomainService;
        _orderDomainService = orderDomainService;
        _telDomainService = telDomainService;
        _workflowDomainService = workflowDomainService;
        _orgRepository = orgRepository;
        _orderRepository = orderRepository;
        _capPublisher = capPublisher;
        _mapper = mapper;
        _orderScreenRepository = orderScreenRepository;
        _orderVisitedDetailRepository = orderVisitedDetailRepository;
        _bulletinRepository = bulletinRepository;
        _circularRepository = circularRepository;
    }

    /// <summary>Handles a notification</summary>
    /// <param name="notification">The notification</param>
    /// <param name="cancellationToken">Cancellation token</param>
    public async Task Handle(EndWorkflowNotify notification, CancellationToken cancellationToken)
    {
        var workflow = notification.Workflow;

        switch (workflow.ModuleCode)
        {
            case WorkflowModuleConsts.KnowledgeAdd://新增知识库
            //case WorkflowModuleConsts.KnowledgeUpdate://修改知识库
            case WorkflowModuleConsts.KnowledgeDelete://删除知识库
                await _knowledgeDomainService.EndWorkKnowledge(workflow, cancellationToken);
                break;

            case WorkflowModuleConsts.TelRestApply:
                await _telDomainService.TelRestApplyPassAsync(workflow.ExternalId, cancellationToken);
                break;

            case WorkflowModuleConsts.OrderHandle:
                var levelOneOrg = workflow.ActualHandleOrgCode.GetHigherOrgCode();
                var org = await _orgRepository.GetAsync(d => d.Id == levelOneOrg, cancellationToken);
                if (org is null)
                    throw new UserFriendlyException($"无效部门编码, levelOneOrg: {levelOneOrg}", "无效部门编码");
                await _workflowDomainService.UpdateOrgLevelOneAsync(workflow, org.Id, org.Name, cancellationToken);

                var order = await _orderDomainService.GetOrderAsync(workflow.ExternalId, cancellationToken);
                order.CheckIfFiled();
                _mapper.Map(workflow, order);
                order.Filed();
                await _orderRepository.UpdateAsync(order, cancellationToken);

                await _capPublisher.PublishAsync(EventNames.HotlineOrderFiled, new OrderFlowDto
                {
                    Order = _mapper.Map<OrderDto>(order),
                    WorkflowTrace = _mapper.Map<WorkflowTraceDto>(notification.Trace)
                }, cancellationToken: cancellationToken);

                break;
            case WorkflowModuleConsts.OrderScreen:
                var screen = await _orderScreenRepository.GetAsync(workflow.ExternalId, cancellationToken);
                if (screen != null)
                {
                    screen.Status = EScreenStatus.End;
                    screen.ReplyContent = workflow.ActualOpinion;
                    await _orderScreenRepository.UpdateAsync(screen, cancellationToken);
                    var visitDetail = await _orderVisitedDetailRepository.GetAsync(screen.VisitDetailId, cancellationToken);
                    if (visitDetail != null)
                    {
                        var screenSatisfy = new Kv() { Key = "6", Value = "甄别满意" };
                        visitDetail.OrgProcessingResults = screenSatisfy;
                        visitDetail.OrgHandledAttitude = screenSatisfy;
                        await _orderVisitedDetailRepository.UpdateAsync(visitDetail, cancellationToken);
                    }
                }
                break;
            case WorkflowModuleConsts.BulletinApply:
                var bulletin = await _bulletinRepository.GetAsync(workflow.ExternalId, cancellationToken);
                if (bulletin != null)
                {
                    bulletin.BulletinState = Share.Enums.Article.EBulletinState.ReviewPass;
                    bulletin.BulletinTime = DateTime.Now;
                    await _bulletinRepository.UpdateAsync(bulletin, cancellationToken);
                }
                break;
            case WorkflowModuleConsts.CircularApply: 
                var circular = await _circularRepository.GetAsync(workflow.ExternalId, cancellationToken);
                if (circular != null)
                {
                    //更新审核状态
                    circular.CircularState = Share.Enums.Article.ECircularState.ReviewPass;
                    circular.CircularTime = DateTime.Now;
                    await _circularRepository.UpdateAsync(circular, cancellationToken);
                    //更新阅读数量

                }
                break;
        }
    }
}